# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
# ====================================================================

require "my-assertions"

require "svn/core"
require "svn/util"

class SvnUtilTest < Test::Unit::TestCase

  def test_to_ruby_const_name
    assert_equal("ABC", Svn::Util.to_ruby_const_name("abc"))
    assert_equal("ABC_DEF", Svn::Util.to_ruby_const_name("abc_def"))
  end

  def test_to_ruby_class_name
    assert_equal("Abc", Svn::Util.to_ruby_class_name("abc"))
    assert_equal("AbcDef", Svn::Util.to_ruby_class_name("abc_def"))
    assert_equal("AbcDef", Svn::Util.to_ruby_class_name("ABC_DEF"))
    assert_equal("XFoo", Svn::Util.to_ruby_class_name("_foo"))
  end
end
